/* -*-C-*-
 * ###################################################################
 *  occ++ - oc to C++ post-compiler for Esterel
 * 
 *  FILE: "occxxUtils.c"
 *                                    created: 03/09/92 00:00:00 
 *                                last update: 27/11/97 12:56:59 
 *  Author: Frdric Boulanger
 *  E-mail: Frederic.Boulanger@supelec.fr
 *    mail: Suplec - Service Informatique
 *          Plateau de Moulon, F-91192 Gif-sur-Yvette cedex
 *     www: http://wwwsi.supelec.fr/fb/fb.html
 *  
 * ========================================================================
 * Permission to use, copy and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that:
 *   - the above copyright notice appears in all copies and,
 *   - both the copyright notice and warranty disclaimer appear in
 *     supporting documentation and,
 *   - any product making use of this software or of parts of this 
 *     software displays the following acknowledgment:
 *       "This product includes software developed by Suplec
 *        and its contributors"
 *     in the documentation or other materials provided with the product 
 *     and in all advertising material mentioning features or use of 
 *     this software.
 *     
 * Warranty disclaimer: this software is provided "as is",
 *   Suplec and Frdric Boulanger disclaim all warranties with regard 
 *   to this software, including all implied warranties of merchantability 
 *   and fitness.
 *   In no event shall Suplec or Frdric Boulanger be liable for any 
 *   special, indirect or consequential damages or any damages whatsoever 
 *   resulting from loss of use, data or profits, whether in an action of 
 *   contract, negligence or other tortuous action, arising out of or in 
 *   connection with the use or performance of this software.
 * ========================================================================
 *  See header file for further information
 * ###################################################################
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "env.h"
#include "occxxErrors.h"
#include "occxxUtils.h"
#include "occxxTables.h"
#include "occxx.h"

static char tmpStr[256];  /* Buffer for string manipulations */

/***********************************************************************/
/* Function dateAndTime returns a pointer to a string representing the */
/* current date and time in the form :                                 */
/*    "DayOfWeek Month DayOfMonth Year at Hour:Minutes:Seconds"        */
/***********************************************************************/
char *dateAndTime() {
  time_t theClock;
  
  theClock = time(&theClock);
  strftime(tmpStr,256,"%A %B %d %Y at %H:%M:%S", localtime(&theClock));
           /* 'man strftime' will teach you a lot about this function! */
  return tmpStr;
}

/**************************************************************************/
/* Function fileName returns a pointer to the file name part of a full    */
/* file name. It looks at the last occurrence of the directory            */
/* separator character (dirSep, defined in env.h) and returns a pointer   */
/* to the next character in the string if it finds it. If not, it returns */
/* the full file name.                                                    */
/**************************************************************************/
char *fileName(char *fullFileName) {
  char *tmp;
  
  tmp = strrchr(fullFileName, dirSep);
  if (tmp == NULL)      /* If there is no directory separator in file name */
    tmp = fullFileName; /* return the file name */
  else                  /* If there is one */
    tmp++;              /* the file name begins at the next character. */
  return tmp;
}

/***********************************************************************/
/* Function checkAlloc allocates memory for nItems of size itemSize.   */
/* It checks for success of the operation and exits through fatalError */
/* if the allocation fails.                                            */
/***********************************************************************/
char *checkAlloc(size_t nItems, size_t itemSize) {
  char *storage;
  
  storage = calloc(nItems,itemSize);
  if (storage == 0) fatalError("Out of memory");
  return storage;
}

/************************************************************************/
/* Function sysTypeName is the special case for predefined functions of */
/* the typeName function. It is only used by the typeName function and  */
/* is not part of the interface of this file.                           */
/************************************************************************/
char *sysTypeName(int typeNum) {
  if (typeNum >= nbSysTypes) {   /* If there is no type with that number -> error */
    sprintf(tmpStr,"System type number %d is out of range (%d)",typeNum,nbSysTypes);
    intError(tmpStr);
  }
  return sysTypeTable[typeNum].name;/* else get the name from the table of types. */
}

/*************************************************************************/
/* Function userTypeName is the special case for user types of           */
/* the typeName function. It is only used by the typeName function and   */
/* is not part of the interface of this file.                            */
/*************************************************************************/
char *userTypeName(int typeNum) {
  if (typeNum >= nbTypes) {  /* If there is no type with that number -> error */
    sprintf(tmpStr,"User type number %d is out of range (%d)",typeNum,nbTypes);
    intError(tmpStr);
  }
  return typeTable[typeNum].name;/* else get the name from the table of types. */
}

/*****************************************************************/
/* Function typeName returns a pointer to the name of a type.    */
/* It handles both cases of a predefined type (negative typeNum) */
/* and of a user defined type (positive typeNum) by calling      */
/* sysTypeName or userTypeName.                                  */
/*****************************************************************/
char *typeName(int typeNum) {
	if (typeNum == -1)
	  return "void";
  if (typeNum < 0)                    /* If it is a predefined type number */
    return sysTypeName(-(typeNum+2)); /* Look in the system types table */
  else                                /* If it is a user defined type number */
    return userTypeName(typeNum);     /* Look in the user types table. */
}


/************************************************************************/
/* Function sysFuncName is the special case for predefined functions of */
/* the funcName function. It is only used by the funcName function and  */
/* is not part of the interface of this file.                           */
/************************************************************************/
char *sysFuncName(int funcNum) {
  if (funcNum >= nbSysFuncs) {   /* If there is no function with that number -> error */
    sprintf(tmpStr,"System function number %d is out of range (%d)",funcNum,nbSysFuncs);
    intError(tmpStr);
  }
  return sysFuncTable[funcNum]->name;/* else get the name from the table of functions */
}

/*************************************************************************/
/* Function userFuncName is the special case for predefined functions of */
/* the funcName function. It is only used by the funcName function and   */
/* is not part of the interface of this file.                            */
/*************************************************************************/
char *userFuncName(int funcNum) {
  if (funcNum >= nbFunctions) { /* If there is no function with that number -> error */
    sprintf(tmpStr,"User function number %d is out of range (%d)",funcNum,nbFunctions);
    intError(tmpStr);
  }
  return funcTable[funcNum].name; /* else, get the name from the table of functions. */
}

/************************************************************/
/* Function funcName returns a pointer to the name of the   */
/* function number funcNum. It handles both cases of a      */
/* predefined function (by calling sysFuncName) and of a    */
/* user defined function (by calling userFuncName).         */
/* Predefined functions have a negative number, and their   */
/* offset in the predefined functions table is -(funcNum+2).*/
/* User defined functions have a positive number, which is  */
/* their offset in the user defined functions table.        */
/************************************************************/
char *funcName(int funcNum) {
	if (funcNum == -1)
	  return "void";
  if (funcNum < 0)                    /* If it is a predefined function number */
    return sysFuncName(-(funcNum+2)); /* Look in the system functions table */
  else                                /* If it is a user defined function number */
    return userFuncName(funcNum);     /* Look in the user functions table. */
}

/************************************************************************/
/* Function sysFuncArity is the special case for predefined functions   */
/* of the funcArity function. It is only used by the funcArity function */
/* and is not part of the interface of this file.                       */
/************************************************************************/
int sysFuncArity(int funcNum) {
  if (funcNum >= nbSysFuncs) {    /* If there is no function with that number -> error */
    sprintf(tmpStr,"System function number %d is out of range (%d)",funcNum, nbSysFuncs);
  }
  return sysFuncTable[funcNum]->nbArgs;/* else get the arity from the table of functions */
}

/**************************************************************************/
/* Function userFuncArity is the special case for predefined functions of */
/* the funcArity function. It is only used by the funcArity function and  */
/* is not part of the interface of this file.                             */
/**************************************************************************/
int userFuncArity(int funcNum) {
  if (funcNum >= nbFunctions) { /* If there is no function with that number -> error */
    sprintf(tmpStr,"User function number %d is out of range (%d)",funcNum,nbFunctions);
    intError(tmpStr);
  }
  return funcTable[funcNum].nbArgs; /* else get the arity from the table of functions */
}

/************************************************************/
/* Function funcArity returns the number of arguments taken */
/* by the function number funcNum. It handles both cases of */
/* a predefined function (by calling sysFuncArity) and of a */
/* user defined function (by calling userFuncArity).        */
/* Predefined functions have a negative number, and their   */
/* offset in the predefined functions table is -(funcNum+1).*/
/* User defined functions have a positive number, which is  */
/* their offset in the user defined functions table.        */
/************************************************************/
int funcArity(int funcNum) {
  if (funcNum < 0)                     /* If it is a predefined function number */
    return sysFuncArity(-(funcNum+2)); /* Look in the system functions table */
  else                                 /* If it is a user defined function number */
    return userFuncArity(funcNum);     /* Look in the user functions table. */
}


/************************************************************************/
/* Function sysProcName is the special case for predefined functions of */
/* the procName function. It is only used by the procName function and  */
/* is not part of the interface of this file.                           */
/************************************************************************/
char *sysProcName(int procNum) {
  if (procNum >= nbSysProcs) {   /* If there is no procedure with that number -> error */
    sprintf(tmpStr,"System procedure number %d is out of range (%d)",procNum,nbSysProcs);
    intError(tmpStr);
  }
  return sysProcTable[procNum]->name;/* else get the name from the table of procedures */
}

/*************************************************************************/
/* Function userProcName is the special case for predefined functions of */
/* the procName function. It is only used by the procName function and   */
/* is not part of the interface of this file.                            */
/*************************************************************************/
char *userProcName(int procNum) {
  if (procNum >= nbProcedures) { /* If there is no procedure with that number -> error */
    sprintf(tmpStr,"User procedure number %d is out of range (%d)",procNum,nbProcedures);
    intError(tmpStr);
  }
  return procTable[procNum].name; /* else, get the name from the table of procedures. */
}

/******************************************************************/
/* Function procName returns a pointer to the name of a procedure */
/* It handles both cases of a predefined procedure (with procNum  */
/* negative) and of a user supplied procedure by calling          */
/* sysProcName or userProcName.                                   */
/******************************************************************/
char *procName(int procNum) {
  if (procNum < 0)                    /* If it is a predefined procedure number */
    return sysProcName(-(procNum+2)); /* Look in the system procedures table */
  else                                /* If it is a user defined procedure number */
    return userProcName(procNum);     /* Look in the user procedures table. */
}

/***********************************************************************/
/* Function opFromTag return a pointer to the name of the operator     */
/* corresponding to the tag of the function name funcName. It is used  */
/* to produce infix operator notation for special function calls.      */
/* For example, "==" corresponds to any function name beginining with  */
/* _eq_. Two tables are used by this function : tagName[] is the table */
/* of known tags, and opName[] is the corresponding table of operators.*/
/***********************************************************************/
char *opFromTag(char *funcName) {
  int len, i;
  char tag[10];
  
  len = strchr(funcName+1,'_') - funcName -1;  /* Get the length of the tag */
  strncpy(tag,funcName+1,len);                 /* Make a copy of it */
  tag[len] = '\0';
  i = 0;
  while((i < nbOperators)&&(strcmp(tag, tagName[i]) != 0)){
    i++;                                       /* Look for the tag in the table of tags */
  }
  if (i < nbOperators)                         /* If there is a matching operator */
    return opName[i];                          /* return its name. */
  else {
    sprintf(tmpStr,"Unknown function tag %s for function %s",tag,funcName);
    fatalError(tmpStr);
    return 0;
  }
}

/***********************************************************************/
/* Function opFromName return a pointer to the name of the operator    */
/* corresponding to the tag of the function name funcName. It is used  */
/* to recognize infix operator notation for special function calls.    */
/* For example, "mul_op_double" is recognized as the '*' operator of   */
/* type 'double'. Two tables are used by this function : cPlusTagName[]*/
/* is the table of the tags, and cPlusOpName[] has the corresponding   */
/* operator names.                                                     */
/***********************************************************************/
char *opFromName(char *tag) {
	int i = 0;
	char *t;
	
	while ((t = cPlusTagName[i]) != 0) {
		if (strcmp(t,tag) == 0) {
			return cPlusOpName[i];
		}
		i++;
	}
	return 0;
}

/***********************************************************************/
/* Function opArity return the arity of the C++ operator corresponding */
/* to the tag argument.                                                */
/***********************************************************************/
int opArity(char *tag) {
	int i = 0;
	char *t;
	
	while((t = cPlusTagName[i]) != 0) {
		if (strcmp(t,tag) == 0) {
			return cPlusArity[i];
		}
		i++;
	}
	return -1;
}

/***********************************************************************/
/* Function needsExternalH returns 1 if an external definition         */
/* interface file must be included.                                    */
/***********************************************************************/
int needsExternalH() {
  int i;
  char buf[255];
  
  if (noDefFile == 1)
    return 0;
  if (nbExecs != 0)
    return 1;
  
  for (i = 0; i < nbTypes; i++) {
  	if (strcmp(typeTable[i].name,"int") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"unsigned") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"long") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"unsigned long") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"char") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"unsigned char") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"float") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"double") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"int*") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"unsigned*") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"long*") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"unsigned long*") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"char*") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"unsigned char*") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"float*") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"double*") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"int&") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"unsigned&") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"long&") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"unsigned long&") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"char&") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"unsigned char&") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"float&") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"double&") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"TypeOfThis") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"TypeOfThis*") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,"TypeOfThis&") == 0)
  	  continue;
  	if (strcmp(typeTable[i].name,moduleName) == 0)
  	  continue;
    sprintf(buf,"%s*", moduleName);
  	if (strcmp(typeTable[i].name,buf) == 0)
  	  continue;
    sprintf(buf,"%s&", moduleName);
  	if (strcmp(typeTable[i].name,buf) == 0)
  	  continue;
  	return 1;
  }
  for (i = 0; i < nbFunctions; i++) {
    if (strncmp(funcTable[i].name,"CC_op_",6) == 0) {
    	continue;                  /* C++ operators */
    }
    if (strncmp(funcTable[i].name,"CC_dot_",7) == 0) {
    	continue;                  /* C++ member access */
    }
    if (strncmp(funcTable[i].name,"CC_arrow_",9) == 0) {
    	continue;                  /* C++ pointer member access */
    }
    if (strncmp(funcTable[i].name,"CC_address_",11) == 0) {
    	continue;                  /* C++ address of object */
    }
    if (strncmp(funcTable[i].name,"CC_star_",8) == 0) {
    	continue;                  /* C++ pointer dereference */
    }
    if (strncmp(funcTable[i].name,"CC_cons_",8) == 0) {
    	continue;                  /* C++ constructor */
    }
  	return 1;
  }
  for (i = 0; i < nbProcedures; i++) {
    if (strncmp(procTable[i].name,"CC_op_",6) == 0) {
    	continue;                  /* C++ operators (+=, -=, ++, --, |=, &=, ...)*/
    }
    if (strncmp(procTable[i].name,"_assign_",8) == 0) {
    	continue;                  /* oc built-in name for operator = */
    }
  	return 1;
  }
  return 0;
}

/************************************************************************************/
/* Procedure printModule deciphers the information built during the parse of the oc */
/* source file and prints it to standard input. It is used in verbose mode.         */
/************************************************************************************/
void printModule() {
  int i,j;
  
  printf("\n### occ++ v%s information\n",occxxVers);
  printf("#   Compiling file %s.\n",sourceFile);
  printf("#   %s\n",dateAndTime());
  printf("Version of oc : %d\n",ocVersion);
  printf("Module name: %s.\n",moduleName);
  printf("Submodules: %d.\n",nbInstances);
  for (i = 0; i < nbInstances; i++)
    printf("\t%d : %s, from file: %s\n",i,moduleTable[i].name, moduleTable[i].srcfile);
  printf("Root module: %s (%d)\n\n",moduleTable[rootModule].name,rootModule);

  printf("User defined types: %d\n",nbTypes);
  for(i = 0; i < nbTypes;i++)
    printf("\t%s\n",typeName(i));

  printf("User constants: %d\n",nbConsts);
  for(i = 0;i < nbConsts;i++)
    printf("\t%s %s\n",typeName(constTable[i].type),constTable[i].name);
  printf("User functions: %d\n",nbFunctions);
  for(i = 0; i < nbFunctions; i++) {
    printf("\t%s %s(",typeName(funcTable[i].resType),funcTable[i].name);
    if (funcTable[i].nbArgs > 0) {
      for(j = 0; j < funcTable[i].nbArgs - 1; j++)
        printf("%s,",typeName(funcTable[i].argTypes[j]));
      printf("%s",typeName(funcTable[i].argTypes[funcTable[i].nbArgs - 1]));
    }
    printf(");\n");
  }
  
  printf("User procedures: %d\n",nbProcedures);
  for(i = 0; i < nbProcedures; i++) {
    printf("\t%s (",procTable[i].name);
    if (procTable[i].nbRefArgs > 0) {
      for(j = 0; j < procTable[i].nbRefArgs - 1; j++)
        printf("%s,",typeName(procTable[i].refArgTypes[j]));
      printf("%s",typeName(procTable[i].refArgTypes[procTable[i].nbRefArgs - 1]));
    }
    printf(") (");
    if (procTable[i].nbValArgs > 0) {
      for(j = 0; j < procTable[i].nbValArgs - 1; j++)
        printf("%s,",typeName(procTable[i].valArgTypes[j]));
      printf("%s",typeName(procTable[i].valArgTypes[procTable[i].nbValArgs - 1]));
    }
    printf(");\n");
  }
  
  printf("Signals: %d\n",nbSignals);
  for (i=0;i < nbSignals;i++) {
    printf("\t(%d) %s : ", i, sigTable[i].name);
    switch(sigTable[i].kind) {
      case sInput:
        printf("input");
        break;
      case sInputOutput:
        printf("inputOutput");
        break;
      case sOutput:
        printf("output");
        break;
      case sSensor:
        printf("sensor");
        break;
      case sLocal:
        printf("local");
        break;
      case sException:
        printf("exception");
        break;
      case sReturn:
        printf("return");
        break;
      default:
        printf("unknown kind (%d)",sigTable[i].kind);
        break;
    }
    switch(sigTable[i].type) {
      case sPure:
        printf(", pure");
        break;
      case sSingle:
        printf(", single(store in var. %d)",sigTable[i].varId);
        break;
      case sMultiple:
        printf(", multiple(store in var. %d",sigTable[i].varId);
        printf(", combined with %s)",funcName(sigTable[i].combFunc));
        break;
      default:
        printf(", unknown type (%d)",sigTable[i].type);
        break;
    }
    printf("\n");
  }
  if (nbImplications != 0) {
    for (i = 0; i < nbImplications; i++) {
      printf("Signal %s implies signal %s\n", 
            sigTable[implTable[i].cause].name, 
            sigTable[implTable[i].effect].name);
    }
  }
  if (nbExclusions != 0) {
    for (i = 0; i < nbExclusions; i++) {
      printf("Signals ");
      for (j = 0; j < exclTable[i].nbSigs - 1; j++) {
        printf("%s ",sigTable[exclTable[i].sigs[j]].name);
      }
      printf("and %s are incompatible.\n", 
        sigTable[exclTable[i].sigs[exclTable[i].nbSigs - 1]].name);
    }
  }
  printf("Variables: %d\n",nbVariables);
  for (i=0;i<nbVariables;i++) {
    switch(varTable[i].kind) {
      case vSigBool:
        printf("\t%d : Boolean of signal %s of type %s.", i,
               sigTable[varTable[i].sigNum].name,typeName(varTable[i].type));
        break;
      case vSigVal:
        printf("\t%d : Value of signal %s of type %s.", i,
               sigTable[varTable[i].sigNum].name,typeName(varTable[i].type));
        break;
      case vVarVal:
        printf("\t%d : %s, user variable of type %s.", i,
               varTable[i].name,typeName(varTable[i].type));
        break;
      case vVarCnt:
        printf("\t%d : %s, occurrence counter (integer).",i,varTable[i].name);
        break;
      case vVarWire:
        printf("\t%d : %s, boolean wire.",i, varTable[i].name);
        break;
      case vVarRegister:
        printf("\t%d : %s, boolean register.",i, varTable[i].name);
        break;
      default:
        printf("\t%d : unknown kind (%d).", i, varTable[i].kind);
        break;
    }
    printf("\n");
  }
  printf("Actions: %d\n",nbActions);
  for(i=0;i<nbActions;i++) {
    printf("\t%d: ",i);
    switch(actTable[i].type) {
      case aGoto:
        printf("goto\n");
        break;
      case aAct:
        printf("execute action %d\n",actTable[i].body.simple.sigNum);
        break;
      case aPresent:
        printf("present %s\n",sigTable[actTable[i].body.simple.sigNum].name);
        break;
      case aOutput:
        printf("output %s\n",sigTable[actTable[i].body.simple.sigNum].name);
        break;
      case aCall:
        printf("call %s\n",procName(actTable[i].body.call.procNum));
        break;
      case aCombine:
        printf("combine %s\n",sigTable[actTable[i].body.simple.sigNum].name);
        break;
      case aReset:
        printf("reset %s\n",varTable[actTable[i].body.simple.sigNum].name);
        break;
      case aIf:
        printf("if\n");
        break;
      case aDsz:
        printf("dsz %s\n",varTable[actTable[i].body.simple.sigNum].name);
        break;
      case aSeq:
        printf("sequence of actions.\n");
        break;
      case aStart:
        printf("start exec %d, task %s.\n",
                actTable[i].body.simple.sigNum,
                taskTable[execTable[actTable[i].body.simple.sigNum].task].name);
        break;
      case aKill:
        printf("kill exec %d, task %s.\n",
                actTable[i].body.simple.sigNum,
                taskTable[execTable[actTable[i].body.simple.sigNum].task].name);
        break;
      case aSuspend:
        printf("suspend exec %d, task %s.\n",
                actTable[i].body.simple.sigNum,
                taskTable[execTable[actTable[i].body.simple.sigNum].task].name);
        break;
      case aActivate:
        printf("activate exec %d, task %s.\n",
                actTable[i].body.simple.sigNum,
                taskTable[execTable[actTable[i].body.simple.sigNum].task].name);
        break;
      case aReturn:
        printf("return from exec %d, task %s.\n",
                actTable[i].body.simple.sigNum,
                taskTable[execTable[actTable[i].body.simple.sigNum].task].name);
        break;
      default:
        printf("Unknown: %d\n",actTable[i].type);
        break;
    }
  }
  printf("Dags: %d\n", nbDags);
  for(i=0; i < nbDags; i++) {
    printf("\tDag %d: ", i);
    if (dagTable[i].isOpenDAG)
      printf("open dag\n");
    else
      printf("closed dag\n");
  }
  printf("\n");
  
  printf("States: %d, initial state is %d\n",nbStates, initState);
  for(i=0;i < nbStates;i++) {
    printf("\tState %d:\n",i);
    if (hasAwaitedInfo) {
      printf("\t\tAwaited signals: ");
      if (stateTable[i].nbAwaited == 0)
        printf("none.");
      else {
        for(j=0; j < stateTable[i].nbAwaited; j++) {
          if (j != 0)
            printf(", ");
          printf("%s",sigTable[stateTable[i].awaited[j]].name);
        }
      }
    } else
      printf("\t\tNo information about awaited signals.");
    printf("\n");
  }
  printf("\n");
}
